#!/bin/csh
echo "./Pathseq_Launch.com script"

# Configurable items in cluster.config and job.config
echo "Reading parameters from the cluster.config and job.config"
set para = `awk -f readconfig.awk T0=cluster.config T01=job.config < .empty.lst`

#Create s3cmd profile
awk -f makes3cmdconfig.awk T0=cluster.config < .empty.lst

#############################################################################################
# Environmental variables
#############################################################################################


#Path settings
set login_info = $para[2]
set s3cmdloc = $para[3]

# Hadoop
set hadoopec2_path = $para[4]
set name_cluster = $para[8]

# Cluster configuration -- Job name
set jname = $para[10]

# Cluster configuration --- Max. size
set maxclus = $para[9]

# Reference Genomes S3 bucket
set ref_loc = $para[11]

# Reads S3 bucket it is indicated as S3://ami-test
set reads_loc = $para[13]

set scripts = $para[14]

#Local directory
set local_dir = `pwd`
#############################################################################################


#Hadoop cluster Generation
set rnumb = `awk 'BEGIN {srand();myRandomNumber=sprintf("%d",rand()*1000); print myRandomNumber}'`

cp $local_dir/cluster.config $local_dir/bin/cluster.config

set hadoop=$local_dir"/bin/hadoop-ec2"
$hadoop launch-cluster $name_cluster $maxclus > $name_cluster".tmp"

#Check Master node name
set name_master = `awk -f check_masternode.awk T0=$name_cluster".tmp" < .empty.lst`

echo "Launch-cluster $name_master"

@ n_nodes1=$maxclus
@ tmp=1
# For the data loading
@ n_nodes2 = $n_nodes1 + $tmp
@ n_nodes = $n_nodes1
#echo $n_nodes

echo "Waiting for master to come up......"
sleep 600

rm $name_cluster".clus"
echo "Building hadoop-cluster......Please wait"

@ no_rnodes = 1
while($no_rnodes < $n_nodes)
        echo "Building hadoop-cluster...please wait..."$no_rnodes $name_cluster
	$hadoopec2_path/hadoop-ec2 "exec /usr/local/hadoop-1.0.2/bin/hadoop dfsadmin -report" $name_cluster > $name_cluster".clus"
	/usr/bin/dos2unix $name_cluster".clus"
	/usr/bin/mac2unix $name_cluster".clus"
	set no_rnodes1 = `grep Name: $name_cluster.clus | wc -l`
	@ no_rnodes =$no_rnodes1 
end
printf "Completed cluster building with total number of nodes : %s\n" $no_rnodes

#Login into Hadoop cluster and Upload the scripts into the hadoop cluster
echo "Upload the Scripts"

echo "/usr/bin/scp -i $login_info"
echo "-r $scripts "
echo "root@"
echo "$name_master"
echo ":"
echo "/usr/local/hadoop-1.0.2/ > tmp"

echo "Upload the scripts and unzip and change permissions"
/usr/bin/scp -i $login_info -r $scripts root@$name_master":/root/" > tmp
/usr/bin/scp -i $login_info -r $scripts root@$name_master":/root/"


/usr/bin/scp -i $login_info -r .s3cfg root@$name_master":/root/"

$hadoopec2_path/hadoop-ec2 "exec /usr/bin/unzip -j /root/$scripts -d /root/" $name_cluster

$hadoopec2_path/hadoop-ec2 "exec /bin/chmod 777 /root/*.*" $name_cluster

$hadoopec2_path/hadoop-ec2 "exec /usr/local/hadoop-1.0.2/bin/start-all.sh" $name_cluster

#echo $jname $reads_loc $n_nodes2 $ref_loc

echo "Run the job on host name"
$hadoopec2_path/hadoop-ec2 "exec ls" $name_cluster

$hadoopec2_path/hadoop-ec2 "exec /root/launch.com $jname $reads_loc $n_nodes2 $ref_loc" $name_cluster

#set HOSTNAME=`awk -f hostname.awk T0=$name_cluster".tmp" < .empty.lst`
#echo $HOSTNAME
#echo "ssh -i $login_info root@$HOSTNAME '/root/launch.com $jname $reads_loc $n_nodes2 $ref_loc'" > tmp
#chmod 777 tmp
#./tmp


#$hadoopec2_path/hadoop-ec2 "exec /root/launch.com $jname $reads_loc $n_nodes2 $ref_loc" $name_cluster
#$hadoopec2_path/hadoop-ec2 "exec /usr/bin/nohup /root/master.com $jname $reads_loc $n_nodes2 $ref_loc &" $name_cluster


#echo "ssh -i $login_info root@$HOSTNAME 'nohup /root/master.com $jname $reads_loc $n_nodes2 $ref_loc > /root/out.txt &'" > tmp
#echo "ssh -i $login_info -o StrictHostKeyChecking=no "root@"$HOSTNAME '/usr/bin/nohup /root/master.com $jname $reads_loc $n_nodes2 $ref_loc > log.txt &'" > tmp
#chmod 777 tmp
#./tmp

echo "End of the run"

echo "Please check your output under s3 bucket s3://ami-"$jname"foutput"

echo "For progress, please visit the following URL"

set nameURL=`awk -f access.awk T0=$name_cluster".tmp" < .empty.lst`
echo $nameURL
