/*
 * Author: Aleksandar D. Kostic
 * Date: 11-24-09
 * Usage: 
 * Details: Parses BLAST output text files. For each read, specifies whether there is a hit or not.
 */


import java.io.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class BlastParser {

	private static String caller= "BlastParser ";

	public static void main(String[] args) throws FileNotFoundException {
		String inputFile =   args[0]; //BLAST results
		String outputFile=   args[1];  //output file

		long start = System.currentTimeMillis();

		long totalReadCounter 		= 0;
		long unmappedReadCounter 	= 0;
		long mappedReadCounter		= 0;



		String[] column;
		final String QUERY  = "Query=";   	   	Pattern queryLine= Pattern.compile(QUERY);
		final String NOHIT  = "No hits found";  Pattern noHits= Pattern.compile(NOHIT);

		try{
			BufferedWriter out = new BufferedWriter(new FileWriter(outputFile));
			FileReader fr1=new FileReader(inputFile);
			BufferedReader br=new BufferedReader(fr1);
			String strLine;
			while ((strLine = br.readLine()) != null)   {
				Matcher queryLineMatcher = queryLine.matcher(strLine);
				Matcher noHitMatcher = noHits.matcher(strLine);

				if ( queryLineMatcher.find() ) {

					if ( (totalReadCounter==0) ) {
						column = strLine.split(" ");
						out.write( column[1] + "\t=" );
						totalReadCounter++;
					}else {
						column = strLine.split(" ");
						out.write( "\n" + column[1] + "\t=" );
						totalReadCounter++;
					}
				}

				if ( noHitMatcher.find() ) {
					out.write( "\tNo Hits." );
					unmappedReadCounter++;
				}

			}
			fr1.close();

			mappedReadCounter= totalReadCounter - unmappedReadCounter;

			long elapsedTimeMillis = System.currentTimeMillis()-start;
			float elapsedTimeSec = elapsedTimeMillis/1000F;
			System.out.println(	"Process complete.\n\n" + 
					"Total reads: " + totalReadCounter + 
					"\nMapped reads: " + mappedReadCounter + 
					"\nUnmapped reads: " + unmappedReadCounter + "\n\n" +
					"CPU time: "+elapsedTimeSec);

			out.flush();
			out.close();
		}catch (Exception e){//Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}

	}
	public static String getCaller() {
		return caller;
	}

}
