/*
 * Details: Converts FASTA files to a file usable by RepeatMasker.
 * Author: Chandra Sekhar Pedamallu
 * Date: 10-2-10
 * 
 */

import java.io.*;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class RepeatMaskerFormat {

	private static String caller= "RepeatMaskerFormat ";
	
	public static void main(String args[]) throws FileNotFoundException {

		String inputFile=  args[0]; //currently supports FASTA or FQ1
		String outputFile= args[1]; //FASTA file with shorter name
		String fileFormat= args[2]; //type FASTA or FQ1

		int formatType= -1;
		if ( fileFormat.equals("FASTA") )
			formatType= 0;
		else if ( fileFormat.equals("FQ1") )
			formatType= 1;
		if ( formatType == -1 ) {
			throw new IllegalArgumentException("Enter 'FASTA' or 'FQ1' file format in argument 3");
		}

		long start = System.currentTimeMillis();

		int seqCounter = 0;

		final String REF  = ">";   	   	Pattern refLine= Pattern.compile(REF);

		try{
			
			BufferedWriter out = new BufferedWriter(new FileWriter(outputFile));
			
			FileReader fr1=new FileReader(inputFile);
			BufferedReader br=new BufferedReader(fr1);
	
			String strLine;

			//FASTA
			if ( formatType == 0 ) {
				while ((strLine = br.readLine()) != null)   {
					Matcher refLineMatcher = refLine.matcher(strLine);
					if ( refLineMatcher.find() ) {
						seqCounter++;
						out.write( ">seq_" + seqCounter + "\n");
					} else
						out.write( strLine + "\n" );
				}
			}

			//FQ1
			if ( formatType == 1 ) {
				while ((strLine = br.readLine()) != null)   {
					if(strLine.trim().length() > 0){
						seqCounter++;
						StringTokenizer smp=new StringTokenizer(strLine, "\t");
						int no=0;
						while(smp.hasMoreElements()){
							String token=smp.nextToken();
							
							if(no==0){
								out.write(">seq_" + seqCounter + "\n" );
							}
							else{
								if(no==1){
									out.write(token + "\n" );
								}
							}
							no++;
						}
					
					}					
					
				}

			}
			fr1.close();
			out.close();

			long elapsedTimeMillis = System.currentTimeMillis()-start;
			float elapsedTimeSec = elapsedTimeMillis/1000F;
			System.out.println(seqCounter+" reads in this dataset."+"  Amount of time: "+elapsedTimeSec);

		}catch (Exception e){//Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}
	}
	public static String getCaller() {
		return caller;
	}
}
