#! /bin/bash

# Author bash file for Master
# Created: Chandra Sekhar Pedamallu @ DFCI , The Broad Inst.

# Master script arguments ./master.com <arg1=Jobname> <arg2=S3-bucket for reads> <arg3=clustersize> <arg4=reference genome bucket>
hadooploc="/usr/local/hadoop-1.0.3"
hadoopstream="/usr/local/hadoop-1.0.3/contrib/streaming/hadoop-streaming-1.0.3.jar"

# For creation of Temporary folder in S3 Bucket
output="ami-"$1"-output"
stat="ami-"$1"-stat"
rinput="ami-"$1"-repeati"
routput="ami-"$1"-repeato"
rfoutput="ami-"$1"-foutput"
rpostsub1="ami-"$1"-psub"
rpostsub11="ami-"$1"-psubo"
rpostsub2="ami-"$1"-psubvelvet"
rpostsub22="ami-"$1"-psubvelveto"

echo $output > $hadooploc/config
echo $stat >> $hadooploc/config
echo $rinput >> $hadooploc/config
echo $routput >> $hadooploc/config
echo $rfoutput >> $hadooploc/config
echo $4 >> $hadooploc/config
echo $rpostsub1 >> $hadooploc/config
echo $rpostsub2 >> $hadooploc/config
echo $rpostsub11 >> $hadooploc/config
echo $rpostsub22 >> $hadooploc/config


#########Config file
$hadooploc/bin/hadoop fs -rmr config 
$hadooploc/bin/hadoop fs -mkdir config 
$hadooploc/bin/hadoop fs -put $hadooploc/config config

#########S3 config file
$hadooploc/bin/hadoop fs -rmr s3config 
$hadooploc/bin/hadoop fs -mkdir s3config 
$hadooploc/bin/hadoop fs -put /root/.s3cfg s3config

#Step 0:List of databases and upload into the hdfs
/usr/bin/s3cmd -f -c /root/.s3cfg ls s3://$4/ > /root/lstdatabase.config
/bin/awk -f lst_databases.awk T0=/root/lstdatabase.config T01=Complete.database < empty.awk

$hadooploc/bin/hadoop fs -put /root/Complete.database config
#$hadooploc/bin/hadoop fs -put /root/Maq.database config
#$hadooploc/bin/hadoop fs -put /root/Blast.database config



#Step 1: Upload the Data load input file########################

i=1

while [ $i -le $3 ]
do
	echo $i > $hadooploc/data_$i.local
	(( i++ ))
done


$hadooploc/bin/hadoop fs -mkdir dataload
$hadooploc/bin/hadoop fs -put $hadooploc/data*.local dataload

###############################################################################################


#Step 2:Data for computational subtraction loader on all nodes STARTS###############################################
# remove the output folder
$hadooploc/bin/hadoop fs -rmr load

# Master loader  - Loads data on to all nodes including master nodes
echo "Master data_loader"
time $hadooploc/bin/hadoop jar $hadoopstream -file /root/mapper_data_compsub.py -mapper /root/mapper_data_compsub.py -reducer NONE -input dataload/*.local -output load -jobconf mapred.tasktracker.tasks.maximum=1 -jobconf mapred.task.timeout=7200000 -jobconf mapred.output.compress=false

echo "Master loader completed"

############################################################Data loader on all nodes ENDS###############################################

# remove the s3bucket meant for statistics with name ami-stat

/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$stat/
/usr/bin/s3cmd -f -c /root/.s3cfg mb s3://$stat/

# remove the s3bucket with name ami-output
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$output/
/usr/bin/s3cmd -f -c /root/.s3cfg mb s3://$output/

/usr/bin/s3cmd -f -c /root/.s3cfg get s3://$2/*.local $hadooploc/

$hadooploc/bin/hadoop fs -rmr test
$hadooploc/bin/hadoop fs -mkdir test
$hadooploc/bin/hadoop fs -put $hadooploc/input*.local test


############################################################BWA alignments + Duplicate remover STARTS###############################################
# remove the output folder
$hadooploc/bin/hadoop fs -rmr maq

#Quality Filter + Maq alignments + Duplicate remover
echo "Maq alignments + Duplicate remover"

time $hadooploc/bin/hadoop jar $hadoopstream -file /root/Fas2FQ1.java -file /root/BWAunmapped.java -file /root/mapper_bwaalignment.py -file /root/FQone2Fasta.java -file /root/UniqueIdentifier_premaq.java -file /root/Sam2Fastq.java -file /root/FQone2Fastq.java -file /root/Fastq2FQone.java -file /root/removeduplicates_new.java -file /root/extractUnmapped_Adapterblast.java -file /root/blastxml.java -file /root/blastxml.cc -file /root/MAQunmapped2FQone.java -file /root/MAQunmapped2fastq.java -file /root/RepeatMaskerFormat.java -file /root/ParsedBlastParser.cc -file /root/BlastParser.java -mapper /root/mapper_bwaalignment.py -reducer NONE -input test/*.local -output maq -jobconf mapred.task.timeout=28800000 -jobconf mapred.tasktracker.tasks.maximum=2 mapred.output.compress=false
echo "Maq alignments + Duplicate remover completed"
############################################################BWA alignments + Duplicate remover ENDS###############################################

############################################################Repeatmasker loader data STARTS#######################################################
# remove the repeatman folder
$hadooploc/bin/hadoop fs -rmr repeatman

# remove the s3bucket with name ami-repeato
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$rinput/
/usr/bin/s3cmd -f -c /root/.s3cfg mb s3://$rinput/

#Repeat masker loader
echo "Repeat masker loader"
time /root/repeatmasker_input.py $rinput $stat $output $3
echo "Repeat masker loader completed"
############################################################Repeatmasker loader data ENDS#######################################################

############################################################Repeatmasker + Blast steps STARTS###############################################
# remove the output folder
$hadooploc/bin/hadoop fs -rmr repeat

# remove the s3bucket with name ami-repeato
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$routput/
/usr/bin/s3cmd -f -c /root/.s3cfg mb s3://$routput/

#Run repeat masker
echo " Run repeat masker"
time $hadooploc/bin/hadoop jar $hadoopstream -file /root/FQone2Fastq.java -file /root/RepeatMaskerFormat.java -file /root/ParsedBlastParser.cc -file /root/BlastParser.java -file /root/RepeatMaskerRead.java -file /root/mapper_repeatmasker.py -mapper /root/mapper_repeatmasker.py -reducer NONE -input repeatman/*.man -output repeat  -jobconf mapred.tasktracker.tasks.maximum=2 -jobconf mapred.task.timeout=604800000 -jobconf mapred.output.compress=false
echo "Repeat masker runs completed"
############################################################Repeatmasker + Blast steps ENDS###############################################

############################################################Data for post subtraction loader on all nodes STARTS###############################################
# remove the output folder
$hadooploc/bin/hadoop fs -rmr load

# Master loader  - Loads data on to all nodes including master nodes
echo "Master data_loader"
time $hadooploc/bin/hadoop jar $hadoopstream -file /root/mapper_data_postsub.py -mapper /root/mapper_data_postsub.py -reducer NONE -input dataload/*.local -output load -jobconf mapred.tasktracker.tasks.maximum=2 -jobconf mapred.task.timeout=3600000 -jobconf mapred.output.compress=false

echo "Master loader completed"

############################################################Data for post substraction loader on all nodes ENDS###############################################

############################################################Post-Subtraction STARTS###############################################
# remove the files in /mnt/mfiles/reads folder
/bin/rm -f /mnt/mfiles/reads/*.*

# remove the postsub1man folder
$hadooploc/bin/hadoop fs -rmr postvelvetman
$hadooploc/bin/hadoop fs -rmr postunmappedman
$hadooploc/bin/hadoop fs -rmr postsub
$hadooploc/bin/hadoop fs -rmr postsubvel

# remove the s3bucket with name ami-postsub
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$rpostsub1/
/usr/bin/s3cmd -f -c /root/.s3cfg mb s3://$rpostsub1/
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$rpostsub11/
/usr/bin/s3cmd -f -c /root/.s3cfg mb s3://$rpostsub11/

# remove the s3bucket with name ami-postsubvelver
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$rpostsub2/
/usr/bin/s3cmd -f -c /root/.s3cfg mb s3://$rpostsub2/
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$rpostsub22/
/usr/bin/s3cmd -f -c /root/.s3cfg mb s3://$rpostsub22/

#Postsubtraction loader
echo "Postsubtraction loader"
time /root/postsubtraction_input.py $routput $stat $rpostsub1 $3 $rpostsub2

#Postsubstraction on the Unmapped reads
echo "Postsubstraction on the Unmapped reads"
time $hadooploc/bin/hadoop jar $hadoopstream -file /root/FQone2Fasta.java -file /root/extractFullQuert4BHitTable.java -file /root/extractUnmapped_latest.java -file /root/Fas2FQ1.java -file /root/FQone2Fastq.java -file /root/RepeatMaskerFormat.java -file /root/ParsedBlastParser.cc -file /root/blastxml.java -file /root/blastxml.cc -file /root/BlastParser.java -file /root/RepeatMaskerRead.java -file /root/mapper_postunmapped.py -mapper /root/mapper_postunmapped.py -reducer NONE -input postunmappedman/*.man -output postsub -jobconf mapred.task.timeout=18800000 -jobconf mapred.output.compress=false

echo "Postsubstraction on the contigs"
time $hadooploc/bin/hadoop jar $hadoopstream -file /root/FQone2Fastq.java -file /root/RepeatMaskerFormat.java -file /root/ParsedBlastParser.cc -file /root/blastxml.java -file /root/blastxml.cc -file /root/FQone2Fasta.java -file /root/extractFullQuert4BHitTable.java -file /root/extractUnmapped_latest.java -file /root/BlastParser.java -file /root/RepeatMaskerRead.java -file /root/mapper_postvelvet.py -mapper /root/mapper_postvelvet.py -reducer NONE -input postvelvetman/*.man -output postsubvel -jobconf mapred.task.timeout=18800000 -jobconf mapred.output.compress=false

echo "Postsubtraction completed"
############################################################Post-Subtraction ENDS###############################################




############################################################Final output Statistics ENDS###############################################
/bin/mkdir $hadooploc/output

# remove the s3bucket with name ami-repeato
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$rfoutput/
/usr/bin/s3cmd -f -c /root/.s3cfg mb s3://$rfoutput/

./statistics.com output.html $hadooploc/output $stat $rinput $routput $rpostsub11 $rpostsub22 $rpostsub2 /usr/local/bwa.db /usr/local/maq.db


/usr/bin/s3cmd -f -c /root/.s3cfg put $hadooploc/output/Output.tar s3://$rfoutput/
/usr/bin/s3cmd -f -c /root/.s3cfg put /root/log.txt s3://$rfoutput/


/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$output/
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$stat/
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$rinput/
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$routput/
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$rpostsub1/
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$rpostsub11/
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$rpostsub2/
/usr/bin/s3cmd -f -c /root/.s3cfg rb s3://$rpostsub22/
