#!/bin/csh
##############################################################################
#	Program: Uploading Reference Genomes into the S3Bucket
#	Input: Download from the Reference Genome and Upload them into the S3 Bucket (s3://ami-pathseqref)
#	Input parameters: 	
#	
#	Written: Chandra Sekhar Pedamallu
#	Copyright@DFCI, The Broad Institute
#############################################################################

#@ n_para=$#

echo "Reading environmental variables from cluster.config and job.config"
set para = `awk -f readconfig.awk T0=cluster.config T01=job.config < .empty.lst`
set s3 = $para[3] # S3 Command line runs
set rbucket = $para[11] # S3Bucket for reference genome
set refgenome = $para[12] # Local reference genome directory
set datatype = $para[15]

set localdir = 	`pwd`

# S3 Bucket availability
set pre = `$s3/s3cmd ls s3://$rbucket | wc -l`
set pre1 = `$s3/s3cmd ls s3://$rbucket"-tmp" | wc -l`
echo $pre

#if($n_para < 2) then
#	echo "Usage: Upload_Reference_Genome.com <Local reference genome directoy> <S3 Bucket i.e. ami-test>"
#	echo "Please check the number of arguments"
#	exit
#endif

if($pre > 0) then
	echo "Reference genome S3 Bucket is already available...."$rbucket
	echo "Upload_Reference_Genome.com"
	echo "Please remove it using S3cmd or rename the exisiting Bucket"
	exit
else
	if($pre1 > 0) then
  	        echo "Reference genome S3 Bucket is already available...."$rbucket"-tmp"
		echo "Upload_Reference_Genome.com"
		echo "Please remove it using S3cmd or rename the exisiting Bucket"
		exit
	else
		$s3/s3cmd mb s3://$rbucket"-tmp"
		$s3/s3cmd mb s3://$rbucket
	endif
endif

# Download the reference Genomes from the ftp://ftp.broadinstitute.org/outgoing/Pathseq_public/
mkdir $localdir/$refgenome

# Download the reference Genomes from the ftp://ftp.broadinstitute.org/outgoing/Pathseq_public/
mkdir $localdir/$refgenome"-tmp"

echo " Downloading the reference genome sequences from Broad Inst. server to our local directory"
if($datatype == "RNASEQ") then
	echo $datatype
	wget -nd -r -P $localdir/$refgenome "ftp://ftp.broadinstitute.org/outgoing/Pathseq_public/latest/RNAseq/compsub_new/"
else
	wget -nd -r -P $localdir/$refgenome "ftp://ftp.broadinstitute.org/outgoing/Pathseq_public/latest/DNAseq/compsub_new/"
endif
echo " Completed the downloading"

echo " Downloading the reference genome sequences from Broad Inst. server to our local directory"
wget -nd -r -P $localdir/$refgenome"-tmp" "ftp://ftp.broadinstitute.org/outgoing/Pathseq_public/latest/postsub/"
echo " Completed the downloading"

# Upload to S3 Bucket
echo "Uploading the reference sequences into S3Bucket"
$s3/s3cmd put $localdir/$refgenome/*.* s3://$rbucket
echo "Completed Uploading"

# Upload to S3 Bucket
echo "Uploading the reference sequences into S3Bucket"
$s3/s3cmd put $localdir/$refgenome"-tmp"/*.* s3://$rbucket"-tmp"
echo "Completed Uploading"

# Remove the temporary folders
rm -r $localdir/$refgenome
rm -r $localdir/$refgenome"-tmp"
